@@ -1,5 +1,5 @@
 /*
- * $Id: JXErrorPane.java,v 1.9 2008/10/14 22:32:49 rah003 Exp $
+ * $Id: JXErrorPane.java 3475 2009-08-28 08:30:47Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -22,16 +22,18 @@ package org.jdesktop.swingx;
 
 import java.awt.Component;
 import java.lang.reflect.InvocationTargetException;
+
 import javax.swing.Icon;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JFrame;
 import javax.swing.JInternalFrame;
 import javax.swing.SwingUtilities;
-import org.jdesktop.swingx.error.ErrorReporter;
+
 import org.jdesktop.swingx.error.ErrorInfo;
-import org.jdesktop.swingx.plaf.ErrorPaneUI;
+import org.jdesktop.swingx.error.ErrorReporter;
 import org.jdesktop.swingx.plaf.ErrorPaneAddon;
+import org.jdesktop.swingx.plaf.ErrorPaneUI;
 import org.jdesktop.swingx.plaf.LookAndFeelAddons;
 
 /**
@@ -123,8 +125,7 @@ import org.jdesktop.swingx.plaf.LookAndFeelAddons;
  * <p><code>JXErrorPane</code> may also be configured with a "Report" button which allows
  * the user to send a bug report, typically through email. This is done through
  * the pluggable {@link org.jdesktop.swingx.error.ErrorReporter} class. Simply instantiate
- * {@link org.jdesktop.swingx.error.EmailErrorReporter} or some custom
- * subclass of <code>ErrorReporter</code> and pass the instance into the
+ * some custom subclass of <code>ErrorReporter</code> and pass the instance into the
  * {@link #setErrorReporter} method.</p>
  *
  * <p><code>JXErrorPane</code> can also be used for displaying fatal error messages to
@@ -246,6 +247,7 @@ public class JXErrorPane extends JComponent {
      * @see javax.swing.JComponent#getUIClassID
      * @see javax.swing.UIDefaults#getUI
      */
+    @Override
     public String getUIClassID() {
         return uiClassID;
     }
