@@ -1,5 +1,5 @@
 /*
- * $Id: JXGlassBox.java 3475 2009-08-28 08:30:47Z kleopatra $
+ * $Id: JXGlassBox.java 3555 2009-11-13 10:40:39Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -55,7 +55,10 @@ import javax.swing.Timer;
  * @author Amy Fowler
  * @author Karl George Schaefer
  * @version 1.0
+ * 
+ * @deprecated (since post-1.6) will be removed without replacement (see Issue #1217-swingx)
  */
+@Deprecated
 public class JXGlassBox extends JXPanel {
     private static final int SHOW_DELAY = 30; // ms
     private static final int TIMER_INCREMENT = 10; // ms
