@@ -1,5 +1,5 @@
 /*
- * $Id: JXTaskPaneContainer.java 3447 2009-08-19 07:33:23Z kleopatra $
+ * $Id: JXTaskPaneContainer.java 3569 2009-12-21 16:25:42Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -23,9 +23,6 @@ package org.jdesktop.swingx;
 import java.awt.event.ContainerAdapter;
 import java.awt.event.ContainerEvent;
 
-import javax.swing.JViewport;
-import javax.swing.Scrollable;
-
 import org.jdesktop.swingx.plaf.LookAndFeelAddons;
 import org.jdesktop.swingx.plaf.TaskPaneContainerAddon;
 import org.jdesktop.swingx.plaf.TaskPaneContainerUI;
@@ -119,6 +116,7 @@ public class JXTaskPaneContainer extends JXPanel {
                 repaint();
             }
         });
+        setScrollableHeightHint(ScrollableSizeHint.VERTICAL_STRETCH);
     }
 
     /**
@@ -185,23 +183,23 @@ public class JXTaskPaneContainer extends JXPanel {
         super.remove(group);
     }
 
-    /**
-     * @see Scrollable#getScrollableTracksViewportHeight()
-     */
-    @Override
-    public boolean getScrollableTracksViewportHeight() {
-        if (getParent() instanceof JViewport) {
-            return (((JViewport) getParent()).getHeight() > getPreferredSize().height);
-        } else {
-            return false;
-        }
-    }
-  
-    /**
-     * @see Scrollable#getScrollableTracksViewportWidth()
-     */
-    @Override
-    public boolean getScrollableTracksViewportWidth() {
-        return true;
-    }
+//    /**
+//     * @see Scrollable#getScrollableTracksViewportHeight()
+//     */
+//    @Override
+//    public boolean getScrollableTracksViewportHeight() {
+//        if (getParent() instanceof JViewport) {
+//            return (((JViewport) getParent()).getHeight() > getPreferredSize().height);
+//        } else {
+//            return false;
+//        }
+//    }
+//  
+//    /**
+//     * @see Scrollable#getScrollableTracksViewportWidth()
+//     */
+//    @Override
+//    public boolean getScrollableTracksViewportWidth() {
+//        return true;
+//    }
 }
