@@ -1,5 +1,5 @@
 /*
- * $Id: JXTree.java,v 1.65 2008/12/23 18:12:34 kschaefe Exp $
+ * $Id: JXTree.java 3698 2010-05-11 13:12:30Z kschaefe $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -25,14 +25,11 @@ import java.applet.Applet;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.KeyboardFocusManager;
-import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.Window;
 import java.awt.event.ActionEvent;
-import java.awt.event.MouseEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
-import java.lang.reflect.Method;
 import java.util.Hashtable;
 import java.util.Vector;
 import java.util.logging.Logger;
@@ -52,7 +49,6 @@ import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 import javax.swing.event.TreeModelEvent;
 import javax.swing.event.TreeModelListener;
-import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicTreeUI;
 import javax.swing.tree.DefaultTreeCellRenderer;
 import javax.swing.tree.TreeCellRenderer;
@@ -63,15 +59,18 @@ import javax.swing.tree.TreePath;
 import org.jdesktop.swingx.decorator.ComponentAdapter;
 import org.jdesktop.swingx.decorator.CompoundHighlighter;
 import org.jdesktop.swingx.decorator.Highlighter;
+import org.jdesktop.swingx.plaf.UIDependent;
 import org.jdesktop.swingx.renderer.StringValue;
 import org.jdesktop.swingx.renderer.StringValues;
 import org.jdesktop.swingx.rollover.RolloverProducer;
 import org.jdesktop.swingx.rollover.RolloverRenderer;
 import org.jdesktop.swingx.rollover.TreeRolloverController;
+import org.jdesktop.swingx.rollover.TreeRolloverProducer;
 import org.jdesktop.swingx.search.SearchFactory;
 import org.jdesktop.swingx.search.Searchable;
 import org.jdesktop.swingx.search.TreeSearchable;
 import org.jdesktop.swingx.tree.DefaultXTreeCellEditor;
+import org.jdesktop.swingx.tree.DefaultXTreeCellRenderer;
 
 
 /**
@@ -153,7 +152,7 @@ import org.jdesktop.swingx.tree.DefaultXTreeCellEditor;
  * <h2>Miscellaneous</h2>
  * 
  * <ul>
- * <li> Improved usabilty for editing: guarantees that the tree is the
+ * <li> Improved usability for editing: guarantees that the tree is the
  * focusOwner if editing terminated by user gesture and guards against data
  * corruption if focusLost while editing
  * <li> Access methods for selection colors, for consistency with JXTable,
@@ -173,15 +172,9 @@ import org.jdesktop.swingx.tree.DefaultXTreeCellEditor;
  * 
  */
 public class JXTree extends JTree {
+    @SuppressWarnings("unused")
     private static final Logger LOG = Logger.getLogger(JXTree.class.getName());
     
-    /** outdated stuff around string conversion from model. */
-    @Deprecated
-    private Method conversionMethod = null;
-    @Deprecated
-    private final static Class[] methodSignature = new Class[] {Object.class};
-    @Deprecated
-    private final static Object[] methodArgs = new Object[] {null};
     
     /** Empty int array used in getSelectedRows(). */
     private static final int[] EMPTY_INT_ARRAY = new int[0];
@@ -261,7 +254,7 @@ public class JXTree extends JTree {
      *
      * @param value an Vector of objects that are children of the root.
      */
-    public JXTree(Vector value) {
+    public JXTree(Vector<?> value) {
         super(value);
         init();
     }
@@ -277,7 +270,7 @@ public class JXTree extends JTree {
      *
      * @param value a Hashtable containing objects that are children of the root.
      */
-    public JXTree(Hashtable value) {
+    public JXTree(Hashtable<?, ?> value) {
         super(value);
         init();
     }
@@ -337,10 +330,9 @@ public class JXTree extends JTree {
      * This must be called from each constructor.
      */
     private void init() {
-        // To support delegation of convertValueToText() to the model...
-        // JW: need to set again (is done in setModel, but at call
-        // in super constructor the field is not yet valid)
-        conversionMethod = getValueConversionMethod(getModel());
+        // Issue #1061-swingx: renderer inconsistencies
+        // force setting of renderer
+        setCellRenderer(createDefaultCellRenderer());
         // Issue #233-swingx: default editor not bidi-compliant 
         // manually install an enhanced TreeCellEditor which 
         // behaves slightly better in RtoL orientation.
@@ -353,14 +345,9 @@ public class JXTree extends JTree {
         // setup.
         // JW PENDING need to mimic ui-delegate default re-set?
         // JW PENDING alternatively, cleanup and use DefaultXXTreeCellEditor in incubator
-        TreeCellRenderer xRenderer = getCellRenderer();
-        if (xRenderer instanceof JXTree.DelegatingRenderer) {
-            TreeCellRenderer delegate = ((JXTree.DelegatingRenderer) xRenderer).getDelegateRenderer();
-            if (delegate instanceof DefaultTreeCellRenderer) { 
-                setCellEditor(new DefaultXTreeCellEditor(this, (DefaultTreeCellRenderer) delegate));
-            }   
+        if (getWrappedCellRenderer() instanceof DefaultTreeCellRenderer) {
+            setCellEditor(new DefaultXTreeCellEditor(this, (DefaultTreeCellRenderer) getWrappedCellRenderer()));
         }
-
         // Register the actions that this class can handle.
         ActionMap map = getActionMap();
         map.put("expand-all", new Actions("expand-all"));
@@ -451,7 +438,7 @@ public class JXTree extends JTree {
 
     /**
      * Returns a Searchable for this component, guaranteed to be not null. This 
-     * implementation lazyly creates a TreeSearchable if necessary.
+     * implementation lazily creates a TreeSearchable if necessary.
      *  
      * 
      * @return a not-null Searchable for this component.
@@ -470,7 +457,8 @@ public class JXTree extends JTree {
      * Sets the Searchable for this component. If null, a default 
      * Searchable will be created and used.
      * 
-     * @param searchable the Searchable to use for this component.
+     * @param searchable the Searchable to use for this component, may be null to 
+     *   indicate using the default.
      * 
      * @see #getSearchable()
      */
@@ -661,6 +649,25 @@ public class JXTree extends JTree {
         super.updateUI();
         installSelectionColors();
         updateHighlighterUI();
+        updateRendererEditorUI();
+        invalidateCellSizeCache();
+    }
+
+    
+    /**
+     * Quick fix for #1060-swingx: icons lost on toggling LAF
+     */
+    protected void updateRendererEditorUI() {
+        if (getCellEditor() instanceof UIDependent) {
+            ((UIDependent) getCellEditor()).updateUI();
+        }
+        // PENDING JW: here we get the DelegationRenderer which is not (yet) UIDependent
+        // need to think about how to handle the per-tree icons
+        // anyway, the "real" renderer usually is updated accidentally 
+        // don't know exactly why, added to the comp hierarchy?
+//        if (getCellRenderer() instanceof UIDependent) {
+//            ((UIDependent) getCellRenderer()).updateUI();
+//        }
     }
 
     /**
@@ -669,10 +676,10 @@ public class JXTree extends JTree {
      * <b>Note:</b> this should be done in the UI delegate.
      */
     private void installSelectionColors() {
-        if (isUIResource(getSelectionBackground())) {
+        if (SwingXUtilities.isUIInstallable(getSelectionBackground())) {
             setSelectionBackground(UIManager.getColor("Tree.selectionBackground"));
         }
-        if (isUIResource(getSelectionForeground())) {
+        if (SwingXUtilities.isUIInstallable(getSelectionForeground())) {
             setSelectionForeground(UIManager.getColor("Tree.selectionForeground"));
         }
         
@@ -684,10 +691,10 @@ public class JXTree extends JTree {
      * <b>Note:</b> this should be done in the UI delegate.
      */
     private void uninstallSelectionColors() {
-        if (isUIResource(getSelectionBackground())) {
+        if (SwingXUtilities.isUIInstallable(getSelectionBackground())) {
             setSelectionBackground(null);
         }
-        if (isUIResource(getSelectionForeground())) {
+        if (SwingXUtilities.isUIInstallable(getSelectionForeground())) {
             setSelectionForeground(null);
         }
     }
@@ -695,34 +702,24 @@ public class JXTree extends JTree {
     /**
      * Updates highlighter after <code>updateUI</code> changes.
      * 
-     * @see org.jdesktop.swingx.decorator.UIDependent
+     * @see org.jdesktop.swingx.plaf.UIDependent
      */
     protected void updateHighlighterUI() {
         if (compoundHighlighter == null) return;
         compoundHighlighter.updateUI();
     }
 
-    /**
-     * Checks if the given value should be set by the LAF. <p>
-     * 
-     * PENDING JW: extract as utility method to ??
-     * 
-     * @param value the value to check
-     * @return true if the value is null or of type UIResource, false otherwise.
-     */
-    private boolean isUIResource(Object value) {
-        return (value == null) || (value instanceof UIResource);
-    }
 
 
 //------------------------ Rollover support
     
     /**
-     * Sets a boolean to enable/disable rollover support. 
-     * Installs or uninstalls the RolloverProducer/-Controller and appropriate
-     * listeners if enable or disabled, respectively. 
-     * If enabled, renderers of implementing RolloverRenderer  
-     * show "live" rollover behaviour, f.i. the cursor over LinkModel cells. <p>
+     * Sets the property to enable/disable rollover support. If enabled, the list
+     * fires property changes on per-cell mouse rollover state, i.e. 
+     * when the mouse enters/leaves a list cell. <p>
+     * 
+     * This can be enabled to show "live" rollover behaviour, f.i. the cursor over a cell 
+     * rendered by a JXHyperlink.<p>
      * 
      * The default value is false.
      * 
@@ -739,12 +736,10 @@ public class JXTree extends JTree {
         if (rolloverEnabled == old) return;
         if (rolloverEnabled) {
             rolloverProducer = createRolloverProducer();
-            addMouseListener(rolloverProducer);
-            addMouseMotionListener(rolloverProducer);
+            rolloverProducer.install(this);
             getLinkController().install(this);
         } else {
-            removeMouseListener(rolloverProducer);
-            removeMouseMotionListener(rolloverProducer);
+            rolloverProducer.release(this);
             rolloverProducer = null;
             getLinkController().release();
         }
@@ -752,9 +747,11 @@ public class JXTree extends JTree {
     }
 
     /**
-     * Returns a boolean indicating whether or not rollover behaviour is enabled. 
+     * Returns a boolean indicating whether or not rollover support is enabled. 
      *
-     * @return a boolean indicating whether or not rollover behaviour is enabled. 
+     * @return a boolean indicating whether or not rollover support is enabled. 
+     * 
+     * @see #setRolloverEnabled(boolean)
      */
     public boolean isRolloverEnabled() {
         return rolloverProducer != null;
@@ -796,62 +793,12 @@ public class JXTree extends JTree {
      * Creates and returns the RolloverProducer to use with this tree.
      * <p>
      * 
-     * This implementation assumes a "hit" for rollover if the mouse is anywhere
-     * in the total width of the tree. Additionally, a pressed to the right (but
-     * outside of the label bounds) is re-dispatched as a pressed just inside
-     * the label bounds. This is a first go for #166-swingx.<p>
-     * 
-     * PENDING JW: bidi-compliance of pressed?
-     * 
      * @return <code>RolloverProducer</code> to use with this tree
      * 
      * @see #setRolloverEnabled(boolean)
      */
     protected RolloverProducer createRolloverProducer() {
-        return new RolloverProducer() {
-            @Override
-            public void mousePressed(MouseEvent e) {
-                JXTree tree = (JXTree) e.getComponent();
-                Point mousePoint = e.getPoint();
-              int labelRow = tree.getRowForLocation(mousePoint.x, mousePoint.y);
-              // default selection
-              if (labelRow >= 0) return;
-              int row = tree.getClosestRowForLocation(mousePoint.x, mousePoint.y);
-              Rectangle bounds = tree.getRowBounds(row);
-              if (bounds == null) {
-                  row = -1;
-              } else {
-                  if ((bounds.y + bounds.height < mousePoint.y) || 
-                       bounds.x > mousePoint.x)   {
-                      row = -1;
-                  }
-              }
-              // no hit
-              if (row < 0) return;
-              tree.dispatchEvent(new MouseEvent(tree, e.getID(), e.getWhen(), 
-                      e.getModifiers(), bounds.x + bounds.width - 2, mousePoint.y,
-                      e.getClickCount(), e.isPopupTrigger(), e.getButton()));
-            }
-
-            @Override
-            protected void updateRolloverPoint(JComponent component,
-                    Point mousePoint) {
-                JXTree tree = (JXTree) component;
-                int row = tree.getClosestRowForLocation(mousePoint.x, mousePoint.y);
-                Rectangle bounds = tree.getRowBounds(row);
-                if (bounds == null) {
-                    row = -1;
-                } else {
-                    if ((bounds.y + bounds.height < mousePoint.y) || 
-                            bounds.x > mousePoint.x)   {
-                           row = -1;
-                       }
-                }
-                int col = row < 0 ? -1 : 0;
-                rollover.x = col;
-                rollover.y = row;
-            }
-        };
+        return new TreeRolloverProducer();
     }
 
   
@@ -928,7 +875,6 @@ public class JXTree extends JTree {
      * PENDING: open up for subclasses again?.
      * 
      * @return the CompoundHighlighter assigned to the table.
-     * @see #setCompoundHighlighter(CompoundHighlighter)
      */
     protected CompoundHighlighter getCompoundHighlighter() {
         if (compoundHighlighter == null) {
@@ -1108,7 +1054,8 @@ public class JXTree extends JTree {
      * @return the default cell renderer to use with this tree.
      */
     protected TreeCellRenderer createDefaultCellRenderer() {
-        return new DefaultTreeCellRenderer();
+//        return new DefaultTreeCellRenderer();
+        return new DefaultXTreeCellRenderer();
     }
 
     /**
@@ -1123,6 +1070,9 @@ public class JXTree extends JTree {
      */
     @Override
     public TreeCellRenderer getCellRenderer() {
+        // PENDING JW: something wrong here - why exactly can't we return super? 
+        // not even if we force the initial setting in init?
+//        return super.getCellRenderer();
         return getDelegatingRenderer();
     }
 
@@ -1156,8 +1106,15 @@ public class JXTree extends JTree {
         // == multiple delegation...
         getDelegatingRenderer().setDelegateRenderer(renderer);
         super.setCellRenderer(delegatingRenderer);
+        // quick hack for #1061: renderer/editor inconsistent
+        if ((renderer instanceof DefaultTreeCellRenderer) && 
+                (getCellEditor() instanceof DefaultXTreeCellEditor)) {
+           ((DefaultXTreeCellEditor) getCellEditor()).setRenderer((DefaultTreeCellRenderer) renderer); 
+        }
+        firePropertyChange("cellRenderer", null, delegatingRenderer);
     }
 
+    
     /**
      * A decorator for the original TreeCellRenderer. Needed to hook highlighters
      * after messaging the delegate.<p>
@@ -1290,8 +1247,8 @@ public class JXTree extends JTree {
                     && (row >= 0)) {
                 result = compoundHighlighter.highlight(result,
                         getComponentAdapter(row));
-            }
-
+            } 
+            
             return result;
         }
             
@@ -1310,6 +1267,22 @@ public class JXTree extends JTree {
 
     }
 
+    /**
+     * Invalidates cell size caching in the ui delegate. May do nothing if there's no
+     * safe (i.e. without reflection) way to message the delegate. <p>
+     * 
+     * This implementation calls BasicTreeUI setLeftChildIndent with the old indent if available. 
+     * Beware: clearing the cache is an undocumented implementation side-effect of the 
+     * method. Revisit if we ever should have a custom ui delegate.
+     * 
+     * 
+     */
+    public void invalidateCellSizeCache() {
+        if (getUI() instanceof BasicTreeUI) {
+            BasicTreeUI ui = (BasicTreeUI) getUI();
+            ui.setLeftChildIndent(ui.getLeftChildIndent());
+        }
+    }
     
 //----------------------- edit
     
@@ -1525,90 +1498,9 @@ public class JXTree extends JTree {
      */
     @Override
     public void setModel(TreeModel newModel) {
-        // To support delegation of convertValueToText() to the model...
-        // JW: method needs to be set before calling super
-        // otherwise there are size caching problems
-        conversionMethod = getValueConversionMethod(newModel);
         super.setModel(newModel);
     }
 
-    /**
-     * Tries to find and return a method for Object --> to String conversion on the
-     * model by reflection. Looks for a signature:
-     * 
-     * <pre> <code>
-     *   String convertValueToText(Object);
-     * </code> </pre>
-     * 
-     * 
-     * 
-     * PENDING JW:
-     * <ul> 
-     * <li>check - does this work with restricted permissions?
-     * <li> widened access for testing - do test!
-     * <li> remove - no longer recommended.
-     * </ul>
-     * @param model the model to detect the method
-     * @return the <code> Method </code> or null if the model has no method with
-     *   the expected signature 
-     *   
-     * @deprecated no longer supported, client code should use {@link #getStringAt(int)} or
-     *    {@link #getStringAt(TreePath)}   
-     */
-    @Deprecated
-    protected Method getValueConversionMethod(TreeModel model) {
-        try {
-            return model == null ? null : model.getClass().getMethod(
-                    "convertValueToText", methodSignature);
-        } catch (NoSuchMethodException ex) {
-            LOG.finer("ex " + ex);
-            LOG.finer("no conversionMethod in " + model.getClass());
-        }
-        return null;
-    }
-
-    /**
-     * {@inheritDoc} <p>
-     * 
-     * Overridden to use the TreeModel's String conversion method instead of super.
-     * 
-     * PENDING JW: remove - that's an oldish approach no longer recommended, superceded
-     * by this Tree's getStringAt methods. But note: the call direction is inverted. The
-     * ol' way was to call this method from the DefaultTreeCellRenderer (which was a bad 
-     * idea to start with). The new way is to give the renderer complete control and expose 
-     * the renderer's conversion via the getStringAt(..) api to arbitrary client code.
-     * 
-     * @see #getValueConversionMethod(TreeModel)
-     * @see #getStringAt(int)
-     * @see #getStringAt(TreePath)
-     * 
-     * @deprecated the implementation will change to call super as soon as the reflexive 
-     *    call to the conversion method will be dropped. So deprecate now to draw attention
-     *    to the pending drop.
-     * 
-     */
-    @Deprecated
-    @Override
-    public String convertValueToText(Object value, boolean selected,
-            boolean expanded, boolean leaf, int row, boolean hasFocus) {
-        // Delegate to model, if possible. Otherwise fall back to superclass...
-        if (value != null) {
-            if (conversionMethod == null) {
-                return value.toString();
-            } else {
-                try {
-                    methodArgs[0] = value;
-                    return (String) conversionMethod.invoke(getModel(),
-                            methodArgs);
-                } catch (Exception ex) {
-                    LOG.finer("ex " + ex);
-                    LOG.finer("can't invoke " + conversionMethod);
-                }
-            }
-        }
-        return "";
-    }
-
 
     
 //------------------------------- ComponentAdapter    
@@ -1672,52 +1564,7 @@ public class JXTree extends JTree {
             TreePath path = tree.getPathForRow(row);
             return path.getLastPathComponent();
         }
-
-        /**
-         * {@inheritDoc} <p>
-         * 
-         * JXTree doesn't support filtering/sorting. This implies that
-         * model and view coordinates are the same. So this method is
-         * implemented to call getValueAt(row, column).
-         * 
-         */
-        @Override
-        public Object getFilteredValueAt(int row, int column) {
-            /** TODO: Implement filtering */
-            return getValueAt(row, column);
-        }
         
-        
-        /**
-         * {@inheritDoc} <p>
-         * 
-         * JXTree doesn't support filtering/sorting. This implies that
-         * model and view coordinates are the same. So this method is
-         * implemented to call getValueAt(row, column).
-         * 
-         */
-        @Override
-        public Object getValue() {
-            return getValueAt(row, column);
-        }
-        
-        
-        /**
-         * {@inheritDoc}
-         */
-        @Override
-        public String getFilteredStringAt(int row, int column) {
-            return tree.getStringAt(row);
-        }
-
-        /**
-         * {@inheritDoc}
-         */
-        @Override
-        public String getString() {
-            return tree.getStringAt(row);
-        }
-
         /**
          * {@inheritDoc}
          */
@@ -1726,6 +1573,14 @@ public class JXTree extends JTree {
             return tree.getStringAt(row);
         }
         
+        /**
+         * {@inheritDoc}
+         */
+        @Override
+        public Rectangle getCellBounds() {
+            return tree.getRowBounds(row);
+        }
+        
         /**
          * {@inheritDoc}
          */
@@ -1759,6 +1614,14 @@ public class JXTree extends JTree {
             return tree.getPathForRow(row).getPathCount() - 1;
         }
         
+        /**
+         * {@inheritDoc}
+         */
+        @Override
+        public boolean isHierarchical() {
+            return true;
+        }
+
         /**
          * {@inheritDoc}
          */
@@ -1774,15 +1637,6 @@ public class JXTree extends JTree {
         public boolean isCellEditable(int row, int column) {
             return false;        /** TODO:  */
         }
-
-        /**
-         * {@inheritDoc}
-         */
-        @Override
-        public void setValueAt(Object aValue, int row, int column) {
-            /** TODO:  */
-        }
-        
     }
 
 
