@@ -1,5 +1,5 @@
 /*
- * $Id: AutoCompleteDocument.java,v 1.8 2008/12/05 14:18:13 kschaefe Exp $
+ * $Id: AutoCompleteDocument.java 3699 2010-05-11 19:05:09Z kschaefe $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -22,9 +22,11 @@ package org.jdesktop.swingx.autocomplete;
 
 import static org.jdesktop.swingx.autocomplete.ObjectToStringConverter.DEFAULT_IMPLEMENTATION;
 
-
 import javax.swing.UIManager;
+import javax.swing.event.DocumentEvent;
 import javax.swing.event.DocumentListener;
+import javax.swing.event.EventListenerList;
+import javax.swing.event.UndoableEditEvent;
 import javax.swing.event.UndoableEditListener;
 import javax.swing.text.AttributeSet;
 import javax.swing.text.BadLocationException;
@@ -41,11 +43,111 @@ import org.jdesktop.swingx.util.Contract;
  * It finds and selects matching items using any implementation of the AbstractAutoCompleteAdaptor.
  */
 public class AutoCompleteDocument implements Document {
-    /** Flag to indicate if adaptor.setSelectedItem has been called.
-     * Subsequent calls to remove/insertString should be ignored
-     * as they are likely have been caused by the adapted Component that
-     * is trying to set the text for the selected component.*/
-    boolean selecting=false;
+    private class Handler implements DocumentListener, UndoableEditListener {
+        private final EventListenerList listenerList = new EventListenerList();
+
+        public void addDocumentListener(DocumentListener listener) {
+            listenerList.add(DocumentListener.class, listener);
+        }
+
+        public void addUndoableEditListener(UndoableEditListener listener) {
+            listenerList.add(UndoableEditListener.class, listener);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void removeDocumentListener(DocumentListener listener) {
+            listenerList.remove(DocumentListener.class, listener);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void removeUndoableEditListener(UndoableEditListener listener) {
+            listenerList.remove(UndoableEditListener.class, listener);
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public void changedUpdate(DocumentEvent e) {
+            e = new DelegatingDocumentEvent(AutoCompleteDocument.this, e);
+            
+            // Guaranteed to return a non-null array
+            Object[] listeners = listenerList.getListenerList();
+            // Process the listeners last to first, notifying
+            // those that are interested in this event
+            for (int i = listeners.length-2; i>=0; i-=2) {
+                if (listeners[i]==DocumentListener.class) {
+                    // Lazily create the event:
+                    // if (e == null)
+                    // e = new ListSelectionEvent(this, firstIndex, lastIndex);
+                    ((DocumentListener)listeners[i+1]).changedUpdate(e);
+                }          
+            }
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void insertUpdate(DocumentEvent e) {
+            e = new DelegatingDocumentEvent(AutoCompleteDocument.this, e);
+            
+            // Guaranteed to return a non-null array
+            Object[] listeners = listenerList.getListenerList();
+            // Process the listeners last to first, notifying
+            // those that are interested in this event
+            for (int i = listeners.length-2; i>=0; i-=2) {
+                if (listeners[i]==DocumentListener.class) {
+                    // Lazily create the event:
+                    // if (e == null)
+                    // e = new ListSelectionEvent(this, firstIndex, lastIndex);
+                    ((DocumentListener)listeners[i+1]).insertUpdate(e);
+                }          
+            }
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void removeUpdate(DocumentEvent e) {
+            e = new DelegatingDocumentEvent(AutoCompleteDocument.this, e);
+            
+            // Guaranteed to return a non-null array
+            Object[] listeners = listenerList.getListenerList();
+            // Process the listeners last to first, notifying
+            // those that are interested in this event
+            for (int i = listeners.length-2; i>=0; i-=2) {
+                if (listeners[i]==DocumentListener.class) {
+                    // Lazily create the event:
+                    // if (e == null)
+                    // e = new ListSelectionEvent(this, firstIndex, lastIndex);
+                    ((DocumentListener)listeners[i+1]).removeUpdate(e);
+                }          
+            }
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void undoableEditHappened(UndoableEditEvent e) {
+            e = new UndoableEditEvent(AutoCompleteDocument.this, e.getEdit());
+            
+            // Guaranteed to return a non-null array
+            Object[] listeners = listenerList.getListenerList();
+            // Process the listeners last to first, notifying
+            // those that are interested in this event
+            for (int i = listeners.length-2; i>=0; i-=2) {
+                if (listeners[i]==UndoableEditListener.class) {
+                // Lazily create the event:
+                // if (e == null)
+                // e = new ListSelectionEvent(this, firstIndex, lastIndex);
+                ((UndoableEditListener)listeners[i+1]).undoableEditHappened(e);
+                }          
+            }
+        }
+    }
     
     /**
      * true, if only items from the adaptors's list can be entered
@@ -53,6 +155,14 @@ public class AutoCompleteDocument implements Document {
      */
     protected boolean strictMatching;
     
+    protected final Document delegate;
+    
+    /** Flag to indicate if adaptor.setSelectedItem has been called.
+     * Subsequent calls to remove/insertString should be ignored
+     * as they are likely have been caused by the adapted Component that
+     * is trying to set the text for the selected component.*/
+    boolean selecting = false;
+    
     /**
      * The adaptor that is used to find and select items.
      */
@@ -60,7 +170,7 @@ public class AutoCompleteDocument implements Document {
     
     ObjectToStringConverter stringConverter;
     
-    protected final Document delegate;
+    private final Handler handler;
     
     /**
      * Creates a new AutoCompleteDocument for the given AbstractAutoCompleteAdaptor.
@@ -78,6 +188,9 @@ public class AutoCompleteDocument implements Document {
         this.stringConverter = stringConverter == null ? DEFAULT_IMPLEMENTATION : stringConverter;
         this.delegate = delegate == null ? createDefaultDocument() : delegate;
         
+        handler = new Handler();
+        this.delegate.addDocumentListener(handler);
+        
         // Handle initially selected object
         Object selected = adaptor.getSelectedItem();
         if (selected!=null) setText(this.stringConverter.getPreferredStringForItem(selected));
@@ -134,7 +247,16 @@ public class AutoCompleteDocument implements Document {
         // insert the string into the document
         delegate.insertString(offs, str, a);
         // lookup and select a matching item
-        LookupResult lookupResult = lookupItem(getText(0, getLength()));
+        LookupResult lookupResult;
+        String pattern = getText(0, getLength());
+        
+        if(pattern == null || pattern.length() == 0) {
+            lookupResult = new LookupResult(null, "");
+            setSelectedItem(lookupResult.matchingItem, lookupResult.matchingString);
+        } else {
+            lookupResult = lookupItem(pattern);
+        }
+        
         if (lookupResult.matchingItem != null) {
             setSelectedItem(lookupResult.matchingItem, lookupResult.matchingString);
         } else {
@@ -144,9 +266,12 @@ public class AutoCompleteDocument implements Document {
                 lookupResult.matchingString = adaptor.getSelectedItemAsString();
                 // imitate no insert (later on offs will be incremented by
                 // str.length(): selection won't move forward)
-                offs = offs-str.length();
-                // provide feedback to the user that his input has been received but can not be accepted
-                UIManager.getLookAndFeel().provideErrorFeedback(adaptor.getTextComponent());
+                offs = str == null ? offs : offs - str.length();
+                
+                if (str != null && !str.isEmpty()) {
+                    // provide feedback to the user that his input has been received but can not be accepted
+                    UIManager.getLookAndFeel().provideErrorFeedback(adaptor.getTextComponent());
+                }
             } else {
                 // no item matches => use the current input as selected item
                 lookupResult.matchingItem=getText(0, getLength());
@@ -154,9 +279,13 @@ public class AutoCompleteDocument implements Document {
                 setSelectedItem(lookupResult.matchingItem, lookupResult.matchingString);
             }
         }
+        
         setText(lookupResult.matchingString);
+        
         // select the completed part
-        adaptor.markText(offs+str.length());
+        int len = str == null ? 0 : str.length();
+        offs = lookupResult.matchingString == null ? 0 : offs + len;
+        adaptor.markText(offs);
     }
     
     /**
@@ -262,14 +391,14 @@ public class AutoCompleteDocument implements Document {
      * {@inheritDoc}
      */
     public void addDocumentListener(DocumentListener listener) {
-        delegate.addDocumentListener(listener);
+        handler.addDocumentListener(listener);
     }
 
     /**
      * {@inheritDoc}
      */
     public void addUndoableEditListener(UndoableEditListener listener) {
-        delegate.addUndoableEditListener(listener);
+        handler.addUndoableEditListener(listener);
     }
 
     /**
@@ -346,14 +475,14 @@ public class AutoCompleteDocument implements Document {
      * {@inheritDoc}
      */
     public void removeDocumentListener(DocumentListener listener) {
-        delegate.removeDocumentListener(listener);
+        handler.removeDocumentListener(listener);
     }
 
     /**
      * {@inheritDoc}
      */
     public void removeUndoableEditListener(UndoableEditListener listener) {
-        delegate.removeUndoableEditListener(listener);
+        handler.removeUndoableEditListener(listener);
     }
 
     /**
@@ -363,7 +492,6 @@ public class AutoCompleteDocument implements Document {
         delegate.render(r);
     }
 
-
     /**
      * Returns if only items from the adaptor's list should be allowed to be entered.
      * @return if only items from the adaptor's list should be allowed to be entered
