@@ -1,5 +1,5 @@
 /*
- * $Id: AutoCompletePropertyChangeListener.java 3100 2008-10-14 22:33:10Z rah003 $
+ * $Id: AutoCompletePropertyChangeListener.java 3697 2010-05-07 15:30:40Z kschaefe $
  *
  * Copyright 2008 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -24,8 +24,9 @@ import java.beans.PropertyChangeListener;
 
 /**
  * @author Karl George Schaefer
- *
+ * @deprecated (pre-1.6.1) accidentally exposed non-public API
  */
+@Deprecated
 public interface AutoCompletePropertyChangeListener extends
         PropertyChangeListener {
 
