@@ -1,5 +1,5 @@
 /*
- * $Id: ComponentAdapter.java,v 1.13 2008/03/14 17:10:34 kleopatra Exp $
+ * $Id: ComponentAdapter.java 3683 2010-04-28 15:07:06Z kschaefe $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -21,19 +21,21 @@
 
 package org.jdesktop.swingx.decorator;
 
+import java.awt.Rectangle;
+
 import javax.swing.JComponent;
 
-import org.jdesktop.swingx.renderer.StringValue;
+import org.jdesktop.swingx.renderer.StringValues;
 
 /**
  * Abstract base class for all component data adapter classes. A
- * <code>ComponentAdapter</code> allows a {@link Filter}, {@link Sorter}, or
+ * <code>ComponentAdapter</code> allows a {@link javax.swing.RowFilter}, {@link javax.swing.RowSorter}, or
  * {@link Highlighter} to interact with a {@link #target} component through a
  * common API.
  * 
  * It has two aspects:
  * <ul>
- * <li> interact with the view state for a given data element. The row/cloumn
+ * <li> interact with the view state for a given data element. The row/column
  * fields and the parameterless methods service this aspect. The coordinates are
  * in view coordinate system. Typical clients of this service are
  * HighlightPredicates and Highlighters.
@@ -73,7 +75,7 @@ import org.jdesktop.swingx.renderer.StringValue;
  * exceptions:
  * <ul>
  * <li> {@link #column} in column view coordinates
- * <li> the mapping method {@link #viewToModel(int)} in view coordinates
+ * <li> the mapping method {@link #convertColumnIndexToModel(int)} in view coordinates
  * </ul>
  * 
  * All input row indices are in model coordinates with exactly three exceptions:
@@ -97,7 +99,7 @@ import org.jdesktop.swingx.renderer.StringValue;
  * 
  * @see org.jdesktop.swingx.decorator.HighlightPredicate
  * @see org.jdesktop.swingx.decorator.Highlighter
- * @see org.jdesktop.swingx.decorator.Filter
+ * @see javax.swing.RowFilter
  */
 public abstract class ComponentAdapter {
     public static final Object DEFAULT_COLUMN_IDENTIFIER = "Column0";
@@ -105,7 +107,7 @@ public abstract class ComponentAdapter {
     public int row = 0;
     /** current column in view coordinates. */
     public int column = 0;
-    protected final JComponent	target;
+    protected final JComponent    target;
 
     /**
      * Constructs a ComponentAdapter, setting the specified component as the
@@ -150,32 +152,6 @@ public abstract class ComponentAdapter {
         return identifier != null ? identifier.toString() : null;
     }
 
-    /**
-     * Returns the columns logical name (== identifier) of the column at 
-     * columnIndex in model coordinates.<p>
-     * 
-     * Note: it's up to the implementation to decide for which
-     * columns it returns a name - most will do so for the
-     * subset with isTestable = true.<p>
-     * 
-     * This implementation delegates to getColumnName. <p>
-     * 
-     * PENDING JW: this is deprecated because the type of an identifer
-     * shouldn't be restricted to String. The only way to gently replace this method is
-     * to add another with a different name - which makes that other name suboptimal.
-     * Maybe we should jump for a rude change and let this return an Object. 
-     * 
-     * @param columnIndex in model coordinates, must be valid.
-     * @return the String value of the column identifier at columnIndex
-     *   or null if no identifier set
-     *   
-     * @deprecated use #getColumnIdentifierAt(int)  
-     */
-    @Deprecated
-    public String getColumnIdentifier(int columnIndex) {
-        return getColumnName(columnIndex);
-    }
-    
     
     /**
      * Returns logical identifier of the column at 
@@ -187,6 +163,12 @@ public abstract class ComponentAdapter {
      * 
      * This implementation returns DEFAULT_COLUMN_IDENTIFIER.
      * 
+     * PENDING JW: This method replaces the old getColumnIdentifier(int)
+     * which returned a String which is overly restrictive.
+     * The only way to gently replace this method was
+     * to add this with a different name - which makes this name suboptimal.
+     * Probably should rename again once the old has died out ;-)
+     * 
      * @param columnIndex in model coordinates, must be valid.
      * @return the identifier of the column at columnIndex or null if it has none.
      * @throws ArrayIndexOutOfBoundsException if columnIndex < 0 or columnIndex >= getColumnCount().
@@ -240,7 +222,7 @@ public abstract class ComponentAdapter {
      * @return the number of columns in the target's data model.
      */
     public int getColumnCount() {
-        return 1;	// default for combo-boxes, lists, and trees
+        return 1;    // default for combo-boxes, lists, and trees
     }
 
     /**
@@ -262,16 +244,6 @@ public abstract class ComponentAdapter {
      *          specified row and column
      */
     public abstract Object getValueAt(int row, int column);
-    
-    /**
-     * Sets the value of the target component's cell identified by the
-     * specified row and column in model coordinates.
-     * 
-     * @param aValue the value to set
-     * @param row in model coordinates
-     * @param column in model coordinates
-     */
-    public abstract void setValueAt(Object aValue, int row, int column);
 
     /**
      * Determines whether this cell is editable.
@@ -293,7 +265,7 @@ public abstract class ComponentAdapter {
      * @return true if the column should be included in testing
      */
     public boolean isTestable(int column) {
-        return modelToView(column) >= 0;
+        return convertColumnIndexToView(column) >= 0;
     }
     
 
@@ -309,7 +281,7 @@ public abstract class ComponentAdapter {
      * coordinate transformation. <p>
      * 
      * This implementation messages the StringValue.TO_STRING with the getValue,
-     * subclasses should re-implement and use the api appropriate for the target component type.
+     * subclasses should re-implement and use the API appropriate for the target component type.
      * 
      * @return the String representation of value of the cell identified by this adapter
      * @see #getValueAt(int, int)
@@ -317,7 +289,7 @@ public abstract class ComponentAdapter {
      * @see #getValue(int)
      */
     public String getString() {
-        return StringValue.TO_STRING.getString(getValue());
+        return getString(convertColumnIndexToModel(column));
     }
 
     /**
@@ -346,7 +318,7 @@ public abstract class ComponentAdapter {
      * query the count of visible rows.<p>
      * 
      * This implementation messages the StringValue.TO_STRING with the filteredValue,
-     * subclasses should re-implement and use the api appropriate for the target component type.<p>
+     * subclasses should re-implement and use the API appropriate for the target component type.<p>
      * 
      * PENDING JW: what about null cell values? StringValue has a contract to return a 
      * empty string then, would that be okay here as well?
@@ -354,10 +326,10 @@ public abstract class ComponentAdapter {
      * @param row the row of the cell in view coordinates
      * @param column the column of the cell in model coordinates.
      * @return the String representation of the filtered value of the cell identified by the row
-     * in view coordinate and the column in model coordiantes
+     * in view coordinate and the column in model coordinates
      */
     public String getFilteredStringAt(int row, int column) {
-        return StringValue.TO_STRING.getString(getFilteredValueAt(row, column));
+        return getStringAt(convertRowIndexToModel(row), column);
     }
     
     /**
@@ -373,7 +345,7 @@ public abstract class ComponentAdapter {
      *          specified row and column
      */
     public String getStringAt(int row, int column) {
-        return StringValue.TO_STRING.getString(getValueAt(row, column));
+        return StringValues.TO_STRING.getString(getValueAt(row, column));
     }
     
     /**
@@ -391,7 +363,7 @@ public abstract class ComponentAdapter {
      * @see #getValue(int)
      */
     public Object getValue() {
-        return getValueAt(row, column);
+        return getValue(convertColumnIndexToModel(column));
     }
 
     
@@ -413,7 +385,7 @@ public abstract class ComponentAdapter {
      * Returns the filtered value of the cell identified by the row
      * in view coordinate and the column in model coordinates.
      * 
-     * Note: the asymetry of the coordinates is intentional - clients like
+     * Note: the asymmetry of the coordinates is intentional - clients like
      * Highlighters are interested in view values but might need to access
      * non-visible columns for testing. While it is possible to access 
      * row coordinates different from the current (that is this.row) it is not
@@ -423,16 +395,27 @@ public abstract class ComponentAdapter {
      * @param row the row of the cell in view coordinates
      * @param column the column of the cell in model coordinates.
      * @return the filtered value of the cell identified by the row
-     * in view coordinate and the column in model coordiantes
+     * in view coordinate and the column in model coordinates
      */
-    public abstract Object getFilteredValueAt(int row, int column);
+    public Object getFilteredValueAt(int row, int column) {
+        return getValueAt(convertRowIndexToModel(row), column);
+    }
+    
+    /**
+     * Returns the bounds of the cell identified by this adapter.<p>
+     * 
+     * @return the bounds of the cell identified by this adapter
+     */
+    public Rectangle getCellBounds() {
+        return target.getBounds();
+    }
 
     /**
      * Returns true if the cell identified by this adapter currently has focus.
      * Otherwise, it returns false.
      *
      * @return true if the cell identified by this adapter currently has focus;
-     * 	Otherwise, return false
+     *  Otherwise, return false
      */
     public abstract boolean hasFocus();
 
@@ -441,7 +424,7 @@ public abstract class ComponentAdapter {
      * Otherwise, it returns false.
      *
      * @return true if the cell identified by this adapter is currently selected;
-     * 	Otherwise, return false
+     *  Otherwise, return false
      */
     public abstract boolean isSelected();
 
@@ -460,7 +443,7 @@ public abstract class ComponentAdapter {
      * such components can never be collapsed.
      *
      * @return true if the cell identified by this adapter is currently expanded;
-     * 	Otherwise, return false
+     *     Otherwise, return false
      */
     public boolean isExpanded() {
         return true; // sensible default for JList and JTable
@@ -473,7 +456,7 @@ public abstract class ComponentAdapter {
      * such components can never have children.
      *
      * @return true if the cell identified by this adapter is a leaf node;
-     * 	Otherwise, return false
+     *     Otherwise, return false
      */
     public boolean isLeaf() {
         return true; // sensible default for JList and JTable
@@ -486,7 +469,7 @@ public abstract class ComponentAdapter {
      * such components can never have children.
      *
      * @return true if the cell identified by this adapter displays the hierarchical node;
-     * 	Otherwise, return false
+     *  Otherwise, return false
      */
     public boolean isHierarchical() {
         return false; // sensible default for JList and JTable
@@ -509,11 +492,13 @@ public abstract class ComponentAdapter {
      * other types of target components, this method returns the columnIndex
      * unchanged.
      *
-     * @param columnIndex index of a column in model coordinates
+     * @param columnModelIndex index of a column in model coordinates
      * @return index of the specified column in view coordinates
+     * @deprecated (pre-1.6.1) use {@link #convertColumnIndexToView(int)}
      */
-    public int modelToView(int columnIndex) {
-        return columnIndex; // sensible default for JList and JTree
+    @Deprecated
+    public int modelToView(int columnModelIndex) {
+        return convertColumnIndexToView(columnModelIndex);
     }
 
    /**
@@ -523,11 +508,61 @@ public abstract class ComponentAdapter {
      * types of target components, this method returns the columnIndex
      * unchanged.
      * 
-     * @param columnIndex index of a column in view coordinates
+     * @param columnViewIndex index of a column in view coordinates
+     * @return index of the specified column in model coordinates
+     * @deprecated (pre-1.6.1) use {@link #convertColumnIndexToModel(int)}
+     */
+    @Deprecated
+    public int viewToModel(int columnViewIndex) {
+        return convertColumnIndexToModel(columnViewIndex);
+    }
+    
+    /**
+     * For target components that support multiple columns in their model,
+     * along with column reordering in the view, this method transforms the
+     * specified columnIndex from model coordinates to view coordinates. For all
+     * other types of target components, this method returns the columnIndex
+     * unchanged.
+     *
+     * @param columnModelIndex index of a column in model coordinates
+     * @return index of the specified column in view coordinates
+     */
+    public int convertColumnIndexToView(int columnModelIndex) {
+        return columnModelIndex; // sensible default for JList and JTree
+    }
+    
+    /**
+     * For target components that support multiple columns in their model, along
+     * with column reordering in the view, this method transforms the specified
+     * columnIndex from view coordinates to model coordinates. For all other
+     * types of target components, this method returns the columnIndex
+     * unchanged.
+     * 
+     * @param columnViewIndex index of a column in view coordinates
      * @return index of the specified column in model coordinates
      */
-    public int viewToModel(int columnIndex) {
-        return columnIndex; // sensible default for JList and JTree
+    public int convertColumnIndexToModel(int columnViewIndex) {
+        return columnViewIndex; // sensible default for JList and JTree
+    }
+    
+    /**
+     * Converts a row index in model coordinates to an index in view coordinates.
+     *
+     * @param rowModelIndex index of a row in model coordinates
+     * @return index of the specified row in view coordinates
+     */
+    public int convertRowIndexToView(int rowModelIndex) {
+        return rowModelIndex; // sensible default for JTree
+    }
+    
+    /**
+     * Converts a row index in view coordinates to an index in model coordinates.
+     * 
+     * @param rowViewIndex index of a row in view coordinates
+     * @return index of the specified row in model coordinates
+     */
+    public int convertRowIndexToModel(int rowViewIndex) {
+        return rowViewIndex; // sensible default for JTree
     }
 
     /**
@@ -538,4 +573,4 @@ public abstract class ComponentAdapter {
         target.revalidate();
         target.repaint();
     }
-}
\ No newline at end of file
+}
