@@ -1,5 +1,5 @@
 /*
- * $Id: PatternMatcher.java 542 2005-10-10 18:03:15Z rbair $
+ * $Id: PatternMatcher.java 3674 2010-04-26 10:15:12Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -27,7 +27,9 @@ import java.util.regex.Pattern;
  * Implemented by classes that work with {@link java.util.regex.Pattern} objects.
 
  * @author Ramesh Gupta
+ * @deprecated (since post-1.6) replaced by PatternMatcher in search package
  */
+@Deprecated
 public interface PatternMatcher {
     public Pattern getPattern();
     public void setPattern(Pattern pattern);
