@@ -1,5 +1,5 @@
 /*
- * $Id: BasicErrorPaneUI.java,v 1.21 2008/06/06 20:55:47 rah003 Exp $
+ * $Id: BasicErrorPaneUI.java 3475 2009-08-28 08:30:47Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -31,7 +31,6 @@ import java.awt.GridBagLayout;
 import java.awt.Insets;
 import java.awt.LayoutManager;
 import java.awt.Point;
-import java.awt.Rectangle;
 import java.awt.Window;
 import java.awt.datatransfer.StringSelection;
 import java.awt.datatransfer.Transferable;
@@ -58,6 +57,7 @@ import javax.swing.JEditorPane;
 import javax.swing.JFrame;
 import javax.swing.JInternalFrame;
 import javax.swing.JLabel;
+import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.KeyStroke;
@@ -407,12 +407,13 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
         collapsedHeight = 0;
         Window w = WindowUtils.findWindow(owner);
         JXErrorDialog dlg = null;
-        if (w == null) {
-            dlg = new JXErrorDialog((Dialog)null, pane);
-        } else if (w instanceof Dialog) {
+        if (w instanceof Dialog) {
             dlg = new JXErrorDialog((Dialog)w, pane);
         } else if (w instanceof Frame) {
             dlg = new JXErrorDialog((Frame)w, pane);
+        } else {
+            // default fallback to null
+            dlg = new JXErrorDialog(JOptionPane.getRootFrame(), pane);
         }
         centerWindow(dlg, owner);
         return dlg;
@@ -454,6 +455,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
         return layout;
     }
 
+    @Override
     public Dimension calculatePreferredSize() {
         //TODO returns a Dimension that is either X wide, or as wide as necessary
         //to show the title. It is Y high.
@@ -788,6 +790,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
          * @see #setPreferredSize
          * @see ComponentUI
          */
+        @Override
         public Dimension getPreferredSize() {
             int width = 0;
             int height = 0;
@@ -815,6 +818,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
             this.details = detailComponent;
         }
 
+        @Override
         protected Transferable createTransferable(JComponent c) {
             String text = details.getSelectedText();
             if (text == null || text.equals("")) {
@@ -825,6 +829,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
             return new StringSelection(text);
         }
 
+        @Override
         public int getSourceActions(JComponent c) {
             return TransferHandler.COPY;
         }
@@ -842,18 +847,18 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
             init(p);
         }
 
-    	protected void init(JXErrorPane p) {
+        protected void init(JXErrorPane p) {
             // FYI: info can be null
-    	    setTitle(p.getErrorInfo() == null ? null : p.getErrorInfo().getTitle());
+            setTitle(p.getErrorInfo() == null ? null : p.getErrorInfo().getTitle());
             initWindow(this, p);
-    	}
+        }
     }
 
     private final class JXErrorFrame extends JFrame {
-    	public JXErrorFrame(JXErrorPane p) {
-    	    setTitle(p.getErrorInfo().getTitle());
+        public JXErrorFrame(JXErrorPane p) {
+            setTitle(p.getErrorInfo().getTitle());
                 initWindow(this, p);
-    	}
+        }
     }
 
     private final class JXInternalErrorFrame extends JInternalFrame {
@@ -870,6 +875,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
             };
             closeButton.addActionListener(closeAction);
             addComponentListener(new ComponentAdapter() {
+                @Override
                 public void componentHidden(ComponentEvent e) {
                     //remove the action listener
                     closeButton.removeActionListener(closeAction);
@@ -925,6 +931,7 @@ public class BasicErrorPaneUI extends ErrorPaneUI {
         }
 
         w.addWindowListener(new WindowAdapter() {
+            @Override
             public void windowClosed(WindowEvent e) {
                 //remove the action listener
                 closeButton.removeActionListener(closeAction);
