@@ -1,5 +1,5 @@
 /*
- * $Id: AbstractRenderer.java,v 1.3 2008/10/14 22:31:41 rah003 Exp $
+ * $Id: AbstractRenderer.java 3472 2009-08-27 13:12:42Z kleopatra $
  *
  * Copyright 2007 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -33,9 +33,9 @@ import org.jdesktop.swingx.rollover.RolloverRenderer;
 public abstract class AbstractRenderer 
     implements  RolloverRenderer, StringValue, Serializable {
 
-    protected ComponentProvider componentController;
+    protected ComponentProvider<?> componentController;
 
-    public AbstractRenderer(ComponentProvider provider) {
+    public AbstractRenderer(ComponentProvider<?> provider) {
         if (provider ==  null) {
             provider = createDefaultComponentProvider();
         }
@@ -45,18 +45,18 @@ public abstract class AbstractRenderer
     /**
      * Returns the ComponentProvider used by this renderer.
      * 
-     * @return the ComponentProvider used by this renderer.
+     * @return the ComponentProvider used by this renderer
      */
-    public ComponentProvider getComponentProvider() {
+    public ComponentProvider<?> getComponentProvider() {
         return componentController;
     }
 
     /**
-     * The default ComponentProvider to use if no special .
+     * The default ComponentProvider to use if no special.
      * 
-     * @return
+     * @return the default <code>ComponentProvider</code>
      */
-    protected abstract ComponentProvider createDefaultComponentProvider();
+    protected abstract ComponentProvider<?> createDefaultComponentProvider();
     
 // --------------- implement StringValue    
     
