@@ -1,5 +1,5 @@
 /*
- * $Id: JXButtonTest.java,v 1.2 2008/10/11 20:42:29 rah003 Exp $
+ * $Id: JXButtonTest.java 3473 2009-08-27 13:17:10Z kleopatra $
  *
  * Copyright 2006 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -21,21 +21,13 @@
  */
 package org.jdesktop.swingx;
 
-import java.net.URL;
-
-import javax.swing.Icon;
-import javax.swing.ImageIcon;
-import javax.swing.JLabel;
-
-import org.junit.runner.RunWith;
-import org.junit.runners.JUnit4;
-import org.junit.Test;
-import org.junit.Before;
-import org.junit.After;
-
 
 import junit.framework.TestCase;
 
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.JUnit4;
+
 /**
  * Unit test for <code>JXButton</code>.
  * <p>
