@@ -1,5 +1,5 @@
 /*
- * $Id: JXTableVisualCheck.java 3541 2009-10-20 10:29:30Z kleopatra $
+ * $Id: JXTableVisualCheck.java 3684 2010-04-28 15:35:33Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -36,10 +36,14 @@ import javax.swing.JTable;
 import javax.swing.JToolBar;
 import javax.swing.JViewport;
 import javax.swing.KeyStroke;
+import javax.swing.RowFilter;
 import javax.swing.SortOrder;
 import javax.swing.SwingUtilities;
 import javax.swing.RowSorter.SortKey;
+import javax.swing.event.RowSorterEvent;
+import javax.swing.event.RowSorterListener;
 import javax.swing.event.TableColumnModelEvent;
+import javax.swing.event.RowSorterEvent.Type;
 import javax.swing.table.DefaultTableModel;
 import javax.swing.table.TableCellEditor;
 import javax.swing.table.TableCellRenderer;
@@ -48,7 +52,6 @@ import javax.swing.table.TableModel;
 
 import org.jdesktop.swingx.action.AbstractActionExt;
 import org.jdesktop.swingx.decorator.AbstractHighlighter;
-import org.jdesktop.swingx.decorator.ColorHighlighter;
 import org.jdesktop.swingx.decorator.ComponentAdapter;
 import org.jdesktop.swingx.decorator.HighlightPredicate;
 import org.jdesktop.swingx.decorator.Highlighter;
@@ -58,8 +61,10 @@ import org.jdesktop.swingx.hyperlink.LinkModelAction;
 import org.jdesktop.swingx.renderer.CheckBoxProvider;
 import org.jdesktop.swingx.renderer.DefaultTableRenderer;
 import org.jdesktop.swingx.renderer.HyperlinkProvider;
+import org.jdesktop.swingx.renderer.StringValue;
 import org.jdesktop.swingx.search.SearchFactory;
 import org.jdesktop.swingx.sort.DefaultSortController;
+import org.jdesktop.swingx.sort.SortUtils;
 import org.jdesktop.swingx.table.ColumnFactory;
 import org.jdesktop.swingx.table.DatePickerCellEditor;
 import org.jdesktop.swingx.table.TableColumnExt;
@@ -82,8 +87,8 @@ public class JXTableVisualCheck extends JXTableUnitTest {
       JXTableVisualCheck test = new JXTableVisualCheck();
       try {
 //        test.runInteractiveTests();
-          test.runInteractiveTests("interactive.*FloatingPoint.*");
-//          test.runInteractiveTests("interactive.*Header.*");
+//          test.runInteractiveTests("interactive.*FloatingPoint.*");
+          test.runInteractiveTests("interactive.*Header.*");
 //          test.runInteractiveTests("interactive.*ColumnProp.*");
 //          test.runInteractiveTests("interactive.*Multiple.*");
 //          test.runInteractiveTests("interactive.*RToL.*");
@@ -92,6 +97,7 @@ public class JXTableVisualCheck extends JXTableUnitTest {
           
 //          test.runInteractiveTests("interactive.*Policy.*");
 //        test.runInteractiveTests("interactive.*Rollover.*");
+//        test.runInteractiveTests("interactive.*Revalidate.*");
 //        test.runInteractiveTests("interactive.*UpdateUI.*");
 //        test.runInteractiveTests("interactiveColumnHighlighting");
       } catch (Exception e) {
@@ -110,6 +116,154 @@ public class JXTableVisualCheck extends JXTableUnitTest {
     }
 
     
+    /**
+     * Issue #35-swingx: visual indicators of secondary sort columns
+     * 
+     * Trick by David Hall: use unicode char 
+     * http://forums.java.net/jive/thread.jspa?threadID=71090
+     * 
+     * As he already noted: it's not necessarily pretty looking ;-)
+     */
+    public void interactiveHeaderSecondarySortIndicator() {
+        DefaultTableModel model = new DefaultTableModel(0, 3) {
+
+            /** 
+             * @inherited <p>
+             */
+            @Override
+            public Class<?> getColumnClass(int columnIndex) {
+                if (columnIndex < getColumnCount() - 1) {
+                    return Integer.class;
+                }
+                return super.getColumnClass(columnIndex);
+            }
+            
+        };
+        int min = hexToInt("25A0");
+        int max = hexToInt("25FF");
+        for (int i = min; i <= max; i++) {
+            Object[] row = new Object[3];
+            row[0] = i;
+            row[1] = i;
+            row[2] = (char) i + ""; 
+            model.addRow(row);
+        }
+        final JXTable table = new JXTable();
+        ColumnFactory factory = new ColumnFactory() {
+
+            /** 
+             * @inherited <p>
+             */
+            @Override
+            public TableColumnExt createTableColumn(int modelIndex) {
+                // TODO Auto-generated method stub
+                TableColumnExt tableColumn = new SortAwareTableColumnExt();
+                tableColumn.setModelIndex(modelIndex);
+                return tableColumn;
+            }
+            
+        };
+        table.setColumnControlVisible(true);
+        table.setColumnFactory(factory);
+        table.setModel(model);
+        RowSorterListener l = new RowSorterListener() {
+            
+            @Override
+            public void sorterChanged(RowSorterEvent e) {
+                if (e.getType() == Type.SORT_ORDER_CHANGED) {
+                    List<TableColumn> list = table.getColumns(true);
+                    List<? extends SortKey> sortKeys = new ArrayList<SortKey>(e.getSource().getSortKeys());
+                    // remove primary
+                    List<? extends SortKey> secondary = sortKeys.subList(1, sortKeys.size());
+                    for (TableColumn tableColumn : list) {
+                        if (tableColumn instanceof TableColumnExt) {
+                            SortKey key = SortUtils.getFirstSortKeyForColumn(secondary, tableColumn.getModelIndex());
+                            Object property = null;
+                            if (key != null && SortUtils.isSorted(key.getSortOrder())) {
+                                property = key.getSortOrder();
+                            }
+                            ((TableColumnExt) tableColumn).putClientProperty(SortAwareTableColumnExt.SORT_ORDER_KEY, property);
+                        }
+                    }
+                }
+                
+            }
+        };
+        table.getRowSorter().addRowSorterListener(l);
+        StringValue sv = new StringValue() {
+
+            @Override
+            public String getString(Object value) {
+                return Integer.toHexString(((Integer) value).intValue());
+            }
+            
+        };
+        table.getColumn(1).setCellRenderer(new DefaultTableRenderer(sv, JLabel.RIGHT));
+        JXFrame frame = showWithScrollingInFrame(table, "Geometric shapes");
+        addComponentOrientationToggle(frame);
+    }
+    
+    public static class SortAwareTableColumnExt extends TableColumnExt {
+        
+        public final static String DESCENDING_CHAR = " \u25bf";
+        public final static String ASCENDING_CHAR = " \u25b5";
+        
+        public final static String SORT_ORDER_KEY = "columnExt.SortOrder";
+
+        /**
+         * @inherited <p>
+         */
+        @Override
+        public Object getHeaderValue() {
+            Object header = super.getHeaderValue();
+            Object sortOrder = getClientProperty(SORT_ORDER_KEY);
+            if (SortOrder.ASCENDING == sortOrder) {
+                header = header + ASCENDING_CHAR;
+            } else if (SortOrder.DESCENDING == sortOrder){
+                header = header + DESCENDING_CHAR;
+            }
+            return header;
+        }
+        
+        
+    }
+    
+    private int hexToInt(String s) {
+        return Integer.parseInt(s, 16);
+    }
+
+    /**
+     * Issue #1254-swingx: JXTable not revalidated on update if filter.
+     * 
+     * Core JTable issue 
+     * Problem is that the update might change the visible row count.
+     */
+    public void interactiveRevalidateOnUpdateWithFilter() {
+        String data[][] = { { "satuAA", "Satu", "SATU", "1" },
+                { "duaAAB", "Dua", "DUA", "2" },
+                { "tigaBAA", "Tiga", "TIGA", "3" },
+                { "empatBBA", "Empat", "EMPAT", "4" } };
+        String cols[] = { "col1", "col2", "col3", "col4" };
+
+        final JXTable table = new JXTable(data, cols);
+        RowFilter<TableModel, Integer> tm = RowFilter.regexFilter(
+                ".*AA.*", 0);
+        table.setRowFilter(tm);
+        JXFrame frame = wrapWithScrollingInFrame(table, "Update with RowFilter");
+        Action action = new AbstractAction("filter first row") {
+            
+            boolean hasAA = true;
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                String newValue = hasAA ? "BB" : "AA";
+                hasAA = !hasAA;
+                table.getModel().setValueAt(newValue, 0, 0);
+            }
+        };
+        addAction(frame, action);
+        show(frame);
+    }
+    
     /**
      * Trying to make null biggest value.
      * 
