@@ -1,5 +1,5 @@
 /*
- * $Id: AbstractPainterTest.java,v 1.6 2008/10/11 20:42:32 rah003 Exp $
+ * $Id: AbstractPainterTest.java 3475 2009-08-28 08:30:47Z kleopatra $
  *
  * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -20,16 +20,21 @@
  */
 package org.jdesktop.swingx.painter;
 
+import java.awt.Graphics2D;
+import java.awt.image.BufferedImage;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.util.ArrayList;
+import java.util.List;
+
 import junit.framework.TestCase;
 
-import java.awt.*;
-import java.awt.image.BufferedImage;
-
+import org.jdesktop.beans.ClassSearchUtils;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
-import org.junit.Test;
-import org.junit.Before;
-import org.junit.After;
 
 
 /**
@@ -42,6 +47,7 @@ public class AbstractPainterTest extends TestCase {
     private TestablePainter p;
     private TestableFilter filter;
 
+    @Override
     @Before
        public void setUp() {
         p = new TestablePainter();
@@ -50,6 +56,7 @@ public class AbstractPainterTest extends TestCase {
         filter = new TestableFilter();
     }
 
+    @Override
     @After
        public void tearDown() {
         g.dispose();
@@ -299,6 +306,44 @@ public class AbstractPainterTest extends TestCase {
         f2.filtered = false;
     }
 
+    @Test
+    public void testVisibilityOfOriginallyProtectedMethodsInExtendingClasses() {
+        List<Class<?>> allClasses = ClassSearchUtils.searchClassPath("org.jdesktop.swingx.");
+        List<Class<?>> painters = new ArrayList<Class<?>>();
+        for (Class<?> clazz : allClasses) {
+            try {
+                if (clazz.isAnonymousClass()) {
+                    // skip inner classes as those expose methods for specific purpose (either tests of to overcome some implementation issue)
+                    continue;
+                }
+                painters.add(clazz.asSubclass(AbstractPainter.class));
+            } catch (ClassCastException e) {
+                // ignore
+            } catch (NoClassDefFoundError e) {
+                // ignore
+                // TODO: find out why class loading fails with this exception even if it was found on a classpath earlier
+            }
+        }
+        // collect all protected methods
+        Method[] methods = AbstractPainter.class.getDeclaredMethods();
+        List<Method> mets = new ArrayList<Method>();
+        for (Method m : methods) {
+            if (Modifier.isProtected(m.getModifiers())) {
+                mets.add(m);
+            }
+        }
+        // check that the methods are not made public
+        for (Class<?> clazz : painters) {
+            for (Method m : mets) {
+                try {
+                    assertTrue("Class " + clazz.getName() + " must keep overriden AbstractPainter method " + m.getName() + " visibility intact.", Modifier.isProtected(clazz.getDeclaredMethod(m.getName(), m.getParameterTypes()).getModifiers()));
+                } catch (NoSuchMethodException e) {
+                    // not declared - ignore
+                }
+            }
+        }
+    }
+    
     //tests that compound behaviors, such as caching in compound situations, works
     //I don't extend CompoundPainter because I don't want to test the CompoundPainter
     //itself, just test the general concepts that go into *any* aggregate Painter
@@ -309,6 +354,7 @@ public class AbstractPainterTest extends TestCase {
             this.painters = painters;
         }
 
+        @Override
         protected void validate(Object object) {
             super.validate(object);
             //iterate over all of the painters and query them to see if they
@@ -323,6 +369,7 @@ public class AbstractPainterTest extends TestCase {
             }
         }
 
+        @Override
         protected void doPaint(Graphics2D g, Object obj, int width, int height) {
             super.doPaint(g, obj, width, height);
             for (TestablePainter p : painters) {
