@@ -1,5 +1,5 @@
 /*
- * $Id: RendererVisualCheck.java 3473 2009-08-27 13:17:10Z kleopatra $
+ * $Id: RendererVisualCheck.java 3675 2010-04-26 10:26:15Z kleopatra $
  *
  * Copyright 2006 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -131,7 +131,7 @@ public class RendererVisualCheck extends InteractiveTestCase {
         try {
 //            test.runInteractiveTests();
 //          test.runInteractiveTests(".*CustomIcons.*");
-          test.runInteractiveTests(".*TextArea.*");
+          test.runInteractiveTests(".*Text.*");
 //          test.runInteractiveTests(".*Color.*");
 //          test.runInteractiveTests("interactive.*ColumnControl.*");
         } catch (Exception e) {
@@ -140,6 +140,21 @@ public class RendererVisualCheck extends InteractiveTestCase {
         }
     }
 
+    /**
+     * Requirement: left aligned icon and right aligned text.
+     * http://forums.java.net/jive/thread.jspa?messageID=398448#398448
+     * 
+     * Issue #1309-swingx: WrappingProvider needs option to "highlight" the icon as well.
+     */
+    public void interactiveIconTextAlignment() {
+        ListModel files = createFileListModel();
+        JXList list = new JXList(files);
+        ComponentProvider<?> text = new LabelProvider(StringValues.FILE_NAME, JLabel.TRAILING);
+        WrappingProvider wrapper = new WrappingProvider(IconValues.FILE_ICON, text, true);
+        list.setCellRenderer(new DefaultListRenderer(wrapper));
+        list.addHighlighter(HighlighterFactory.createSimpleStriping());
+        showWithScrollingInFrame(list, "alignment in wrappingProvider");
+    }
     
     /**
      * Issue #897-swingx: Opacity issues of JRendererCheckBox - striping lost.
