@@ -1,5 +1,5 @@
 /*
- * $Id: RenderingTest.java,v 1.40 2008/12/23 18:12:38 kschaefe Exp $
+ * $Id: RenderingTest.java 3473 2009-08-27 13:17:10Z kleopatra $
  *
  * Copyright 2006 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -43,7 +43,7 @@ import org.jdesktop.swingx.JXHyperlink;
 import org.jdesktop.swingx.JXPanel;
 import org.jdesktop.swingx.JXTable;
 import org.jdesktop.swingx.JXTreeTable;
-import org.jdesktop.swingx.hyperlink.LinkAction;
+import org.jdesktop.swingx.hyperlink.AbstractHyperlinkAction;
 import org.jdesktop.swingx.icon.EmptyIcon;
 import org.jdesktop.swingx.painter.ShapePainter;
 import org.jdesktop.swingx.table.TableColumnExt;
@@ -150,7 +150,7 @@ public class RenderingTest extends TestCase {
      */
     @Test
     public void testHyperlinkStringValue() {
-        LinkAction linkAction = new LinkAction() {
+        AbstractHyperlinkAction<?> linkAction = new AbstractHyperlinkAction<Object>() {
 
             public void actionPerformed(ActionEvent e) {
                 // nothing
@@ -269,6 +269,10 @@ public class RenderingTest extends TestCase {
     @Test
     public void testToolTipManagerTreeTableTreeRenderer() {
         JXTreeTable treeTable = new JXTreeTable(new ComponentTreeTableModel(new JXPanel()));
+        // core row conversion is lenient than ol' swingx
+        // PENDING JW: raise an issue about JXTreeTable not being sortable 
+        // (remove the default auto-createRowSorter)
+        treeTable.setRootVisible(true);
         JComponent label = (JComponent) treeTable.prepareRenderer(treeTable.getCellRenderer(0, 0), 0, 0);
         String tip = "some tip";
         PropertyChangeReport report = new PropertyChangeReport();
@@ -286,7 +290,7 @@ public class RenderingTest extends TestCase {
      */
     @Test
     public void testToolTipManagerHyperlinkProvider() {
-        LinkAction linkAction = new LinkAction() {
+        AbstractHyperlinkAction<?> linkAction = new AbstractHyperlinkAction<Object>() {
 
             public void actionPerformed(ActionEvent e) {
                 // do nothing
@@ -654,16 +658,16 @@ public class RenderingTest extends TestCase {
     @Test
     public void testWrappingProviderWrappeeConstructors() {
         IconValue iv = IconValues.FILE_ICON;
-        LabelProvider delegate = new LabelProvider(FormatStringValue.DATE_TO_STRING);
+        LabelProvider delegate = new LabelProvider(StringValues.DATE_TO_STRING);
         assertWrappingProviderState(new WrappingProvider(delegate),
                 null, null, delegate, true);
         assertWrappingProviderState(new WrappingProvider(delegate, false), 
                 null, null, delegate, false);
         assertWrappingProviderState(new WrappingProvider(iv, delegate, false), 
                 iv, null, delegate, false);
-        assertWrappingProviderState(new WrappingProvider(iv, (ComponentProvider) null, false), 
+        assertWrappingProviderState(new WrappingProvider(iv, (ComponentProvider<?>) null, false), 
                 iv, null, null, false);
-        assertWrappingProviderState(new WrappingProvider(null, (ComponentProvider) null, false), 
+        assertWrappingProviderState(new WrappingProvider(null, (ComponentProvider<?>) null, false), 
                 null, null, null, false);
     }
    
@@ -674,7 +678,7 @@ public class RenderingTest extends TestCase {
     @Test
     public void testWrappingProviderWrappeeStringValueConstructors() {
         IconValue iv = IconValues.FILE_ICON;
-        StringValue sv = FormatStringValue.DATE_TO_STRING;
+        StringValue sv = StringValues.DATE_TO_STRING;
         assertWrappingProviderState(new WrappingProvider(iv),
                 iv, null, null, true);
         assertWrappingProviderState(new WrappingProvider(iv, sv), 
@@ -704,9 +708,9 @@ public class RenderingTest extends TestCase {
     }
     
     private void assertWrappingProviderState(WrappingProvider provider, 
-            IconValue iv, StringValue sv, ComponentProvider delegate, boolean unwrap) {
+            IconValue iv, StringValue sv, ComponentProvider<?> delegate, boolean unwrap) {
         if (iv == null) {
-            assertEquals("default StringValue must be empty", StringValue.EMPTY, 
+            assertEquals("default StringValue must be empty", StringValues.EMPTY, 
                     provider.getStringValue());
         } else {
             assertTrue("provider's StringValue must be a MappedValue containing the IconValue, but" +
@@ -848,7 +852,7 @@ public class RenderingTest extends TestCase {
      */
     @Test
     public void testButtonProviderConstructor() {
-        ComponentProvider provider = new CheckBoxProvider();
+        ComponentProvider<?> provider = new CheckBoxProvider();
         assertEquals(JLabel.CENTER, provider.getHorizontalAlignment());
         assertEquals(StringValues.EMPTY, provider.getStringValue());
        
@@ -975,7 +979,7 @@ public class RenderingTest extends TestCase {
         assertEmptyContext(new HyperlinkProvider());
     }
     
-    private void assertEmptyContext(ComponentProvider provider) {
+    private void assertEmptyContext(ComponentProvider<?> provider) {
         DefaultListRenderer renderer = new DefaultListRenderer(provider);
         renderer.getListCellRendererComponent(null, null, -1, false, false);
         // treeRenderer - use the same provider, can't do in real life, 
@@ -1000,7 +1004,7 @@ public class RenderingTest extends TestCase {
         // wrong assumption - we are wrapping...
 //        assertSame(FormatStringValue.DATE_TO_STRING, renderer.componentController.formatter);
         assertSame(StringValues.DATE_TO_STRING, ((WrappingProvider) renderer.componentController).wrappee.formatter);
-        ComponentProvider controller = new CheckBoxProvider();
+        ComponentProvider<?> controller = new CheckBoxProvider();
         renderer = new DefaultTreeRenderer(controller);
         assertSame(controller, renderer.componentController);
     }
@@ -1016,11 +1020,49 @@ public class RenderingTest extends TestCase {
         renderer = new DefaultListRenderer(StringValues.DATE_TO_STRING);
         assertTrue(renderer.componentController instanceof LabelProvider);
         assertSame(StringValues.DATE_TO_STRING, renderer.componentController.formatter);
-        ComponentProvider controller = new CheckBoxProvider();
+        ComponentProvider<?> controller = new CheckBoxProvider();
         renderer = new DefaultListRenderer(controller);
         assertSame(controller, renderer.componentController);
     }
 
+    /**
+     * Issue 970-swingx: add convenience constructors with IconValues
+     * Test doc'ed constructor behaviour of default list renderer.
+     *
+     */
+    @Test
+    public void testDefaultListRendererConstructorWithIconValue() {
+        DefaultListRenderer renderer = new DefaultListRenderer(StringValues.TO_STRING, IconValues.ICON);
+        assertTrue(renderer.componentController instanceof LabelProvider);
+        LabelProvider provider = (LabelProvider) renderer.componentController;
+        ImageIcon icon = (ImageIcon) XTestUtils.loadDefaultIcon();
+        icon.setDescription("description");
+        TableCellContext context = new TableCellContext();
+        context.value = icon;
+        JLabel label = provider.getRendererComponent(context);
+        assertEquals(icon, label.getIcon());
+        assertEquals(StringValues.TO_STRING.getString(icon), label.getText());
+    }
+
+    /**
+     * Issue #970-swingx: add convenience constructors with IconValues
+     * Test doc'ed constructor behaviour of default list renderer.
+     *
+     */
+    @Test
+    public void testDefaultListRendererConstructorWithIconValueAndAlign() {
+        DefaultListRenderer renderer = new DefaultListRenderer(StringValues.TO_STRING, IconValues.ICON, JLabel.TRAILING);
+        assertTrue(renderer.componentController instanceof LabelProvider);
+        LabelProvider provider = (LabelProvider) renderer.componentController;
+        ImageIcon icon = (ImageIcon) XTestUtils.loadDefaultIcon();
+        icon.setDescription("description");
+        TableCellContext context = new TableCellContext();
+        context.value = icon;
+        JLabel label = provider.getRendererComponent(context);
+        assertEquals(icon, label.getIcon());
+        assertEquals(StringValues.TO_STRING.getString(icon), label.getText());
+        assertEquals(JLabel.TRAILING, label.getHorizontalAlignment());
+    }
     /**
      * Test doc'ed constructor behaviour of default table renderer.
      *
@@ -1032,11 +1074,50 @@ public class RenderingTest extends TestCase {
         renderer = new DefaultTableRenderer(StringValues.DATE_TO_STRING);
         assertTrue(renderer.componentController instanceof LabelProvider);
         assertSame(StringValues.DATE_TO_STRING, renderer.componentController.formatter);
-        ComponentProvider controller = new CheckBoxProvider();
+        ComponentProvider<?> controller = new CheckBoxProvider();
         renderer = new DefaultTableRenderer(controller);
         assertSame(controller, renderer.componentController);
     }
 
+    /**
+     * Issue #970-swingx: text constructors with IconValues
+     * Test doc'ed constructor behaviour of default table renderer.
+     *
+     */
+    @Test
+    public void testDefaultTableRendererConstructorWithIconValue() {
+        DefaultTableRenderer renderer = new DefaultTableRenderer(StringValues.TO_STRING, IconValues.ICON);
+        assertTrue(renderer.componentController instanceof LabelProvider);
+        LabelProvider provider = (LabelProvider) renderer.componentController;
+        ImageIcon icon = (ImageIcon) XTestUtils.loadDefaultIcon();
+        icon.setDescription("description");
+        TableCellContext context = new TableCellContext();
+        context.value = icon;
+        JLabel label = provider.getRendererComponent(context);
+        assertEquals(icon, label.getIcon());
+        assertEquals(StringValues.TO_STRING.getString(icon), label.getText());
+    }
+
+    /**
+     * Issue #970-swingx: text constructors with IconValues
+     * Test doc'ed constructor behaviour of default table renderer.
+     *
+     */
+    @Test
+    public void testDefaultTableRendererConstructorWithIconValueAndAlign() {
+        DefaultTableRenderer renderer = new DefaultTableRenderer(StringValues.TO_STRING, IconValues.ICON, JLabel.TRAILING);
+        assertTrue(renderer.componentController instanceof LabelProvider);
+        LabelProvider provider = (LabelProvider) renderer.componentController;
+        ImageIcon icon = (ImageIcon) XTestUtils.loadDefaultIcon();
+        icon.setDescription("description");
+        TableCellContext context = new TableCellContext();
+        context.value = icon;
+        JLabel label = provider.getRendererComponent(context);
+        assertEquals(icon, label.getIcon());
+        assertEquals(StringValues.TO_STRING.getString(icon), label.getText());
+        assertEquals(JLabel.TRAILING, label.getHorizontalAlignment());
+    }
+
     /**
      * public methods of <code>ComponentProvider</code> must cope
      * with null context. Here: test getRenderingComponent in WrappingProvider.
@@ -1054,7 +1135,7 @@ public class RenderingTest extends TestCase {
      */
     @Test
     public void testGetComponentNullContext() {
-        ComponentProvider controller = new LabelProvider();
+        ComponentProvider<?> controller = new LabelProvider();
         assertEquals(controller.rendererComponent, controller.getRendererComponent(null));
     }
     
