@@ -1,5 +1,5 @@
 /*
- * $Id: TableRendererTest.java,v 1.10 2008/10/11 20:42:32 rah003 Exp $
+ * $Id: TableRendererTest.java 3473 2009-08-27 13:17:10Z kleopatra $
  *
  * Copyright 2006 Sun Microsystems, Inc., 4150 Network Circle,
  * Santa Clara, California 95054, U.S.A. All rights reserved.
@@ -44,11 +44,11 @@ import org.jdesktop.swingx.InteractiveTestCase;
 import org.jdesktop.swingx.JXTable;
 import org.jdesktop.swingx.test.XTestUtils;
 import org.jdesktop.test.SerializableSupport;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
-import org.junit.Test;
-import org.junit.Before;
-import org.junit.After;
 
 
 /**
@@ -92,6 +92,53 @@ public class TableRendererTest extends InteractiveTestCase {
         table.getColumnModel().getColumn(xColumn).setCellRenderer(xTableRenderer);
     }
     
+    /**
+     * Issue #484-swingx: dnd on color not showing.
+     * Can't really test - can't mock a drop-on only bare context methods.
+     */
+    @Test
+    public void testCellContextDropOn() {
+        TableCellContext cellContext = new TableCellContext();
+        cellContext.installContext(null, "whatever", 0, 0, false, true, false, false);
+        assertFalse("context must cope with null component", cellContext.isDropOn());
+        // fake ... and test internals
+        cellContext.dropOn = true;
+        assertTrue("context must use dropOn flag ... dooh", cellContext.isDropOn());
+    }
+    /**
+     * Issue #484-swingx: dnd on color not showing.
+     * Can't really test - can't mock a drop-on only bare context methods.
+     */
+    @Test
+    public void testCellContextDropOnColors() {
+        TableCellContext cellContext = new TableCellContext();
+        Color dropBackground = UIManager.getColor("Table.dropCellBackground");
+        LOG.info("background " + dropBackground);
+        if (dropBackground != null) {
+            assertEquals(dropBackground, cellContext.getDropCellBackground());
+        }
+        Color dropForeground = UIManager.getColor("Table.dropCellForeground");
+        LOG.info("foreground " + dropForeground);
+        if (dropForeground != null) {
+            assertEquals(dropForeground, cellContext.getDropCellForeground());
+        }
+    }
+    /**
+     * Issue ??-swingx: getEditable throws if cell coordinates invalid
+     */
+    @Test
+    public void testCellContextEditable() {
+        TableCellContext cellContext = new TableCellContext();
+        cellContext.installContext(null, "whatever", 0, 0, false, true, false, false);
+        assertFalse("context must cope with null component", cellContext.isEditable());
+        JXTable table = new JXTable();
+        cellContext.installContext(table, "whatever", 0, 0, false, true, false, false);
+        // KEEP commented lines is real-world example that'll blow
+//        DefaultVisuals visuals = new DefaultVisuals();
+//        visuals.configureVisuals(new JLabel(), cellContext);
+        assertFalse("context must cope with invalid coordinates", cellContext.isEditable());
+    }
+    
     /**
      * Test constructors: here convenience with alignment and converter
      *
@@ -138,7 +185,7 @@ public class TableRendererTest extends InteractiveTestCase {
      */
     private TableModel createTableModelWithDefaultTypes() {
         String[] names = {"Object", "Number", "Double", "Date", "ImageIcon", "Boolean"};
-        final Class[] types = {Object.class, Number.class, Double.class, Date.class, ImageIcon.class, Boolean.class};
+        final Class<?>[] types = {Object.class, Number.class, Double.class, Date.class, ImageIcon.class, Boolean.class};
         DefaultTableModel model = new DefaultTableModel(names, 0) {
 
             @Override
