@@ -30,22 +30,28 @@ import java.math.BigInteger;
 import java.text.Format;
 import java.text.NumberFormat;
 import java.text.ParseException;
+import java.util.Date;
 import java.util.logging.Logger;
 
+import javax.swing.AbstractAction;
+import javax.swing.Action;
 import javax.swing.Box;
 import javax.swing.InputVerifier;
 import javax.swing.JCheckBox;
 import javax.swing.JComponent;
 import javax.swing.JFormattedTextField;
+import javax.swing.KeyStroke;
 import javax.swing.event.DocumentEvent;
 import javax.swing.event.DocumentListener;
 import javax.swing.table.DefaultTableModel;
+import javax.swing.table.TableCellEditor;
 import javax.swing.text.NumberFormatter;
 
 import org.jdesktop.swingx.InteractiveTestCase;
 import org.jdesktop.swingx.JXFrame;
 import org.jdesktop.swingx.JXTable;
 import org.jdesktop.swingx.JXTable.NumberEditor;
+import org.jdesktop.test.CellEditorReport;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -77,14 +83,67 @@ public class NumberEditorExtTest extends InteractiveTestCase {
     }
 
     private static final int INTEGER_COLUMN = 0;
-    /** a table with a model which has column class Integer in INTEGER_COLUMN. */
+    private static final int NUMBER_COLUMN = 1;
+    private static final int OBJECT_COLUMN = 2;
+    private static final int COLUMN_COUNT = OBJECT_COLUMN + 1;
+    
+    /** a table with a model which has column class Integer in INTEGER_COLUMN and
+     * column class Number in NUMBER_COLUMN. */
     private JXTable table;
     /** a NumberEditorExt configured with IntegerFormat. */
     private NumberEditorExt cellEditor;
 
-
+    /** a NumberEditorExt configured with strict IntegerFormat. */
     private NumberEditorExt cellEditorStrict;
+    
+    /**
+     * Issue #1293-swingx: NumberEditorExt removes Escape binding for all formatted text fields.
+     */
+    @Test
+    public void testFormattedTextFieldHasEscapeBinding() {
+        JFormattedTextField field = new JFormattedTextField(new Date());
+        KeyStroke keyStroke = KeyStroke.getKeyStroke("ESCAPE");
+        assertNotNull("text field must have ESCAPE binding", field.getInputMap().get(keyStroke ));
+    }
+    
+    
+    
+    /**
+     * Issue #1236-swingx: NumberEditor throws for non-number columns.
+     * 
+     * Strict editor delegates to formatter -  which defaults to string valueclass.
+     */
+    @Test 
+    public void testStrictIsDefault() {
+        TableCellEditor editor = table.getDefaultEditor(Number.class);
+        assertTrue("sanity: expect NumberEditorExt but was " + editor, editor instanceof NumberEditorExt);
+        assertEquals("default formatter is strict", true, 
+                ((NumberEditorExt) editor).hasStrictFormatter());
+    }
+    
+    /**
+     * Issue #1236-swingx: NumberEditor throws for non-number columns.
+     * 
+     * Strict editor delegates to formatter -  which defaults to string valueclass.
+     */
+    @Test 
+    public void testEditorStrictObjectColumn() {
+        cellEditorStrict.getTableCellEditorComponent(table, null, false, 0, OBJECT_COLUMN);
+    }
+    
+    /**
+     * Issue #1236-swingx: NumberEditor throws for non-number columns.
+     * 
+     * Old default: non-strict editor throws IllegalState if column type
+     * is not a subtype of Number.
+     */
+    @Test (expected = IllegalStateException.class)
+    public void testEditorObjectColumn() {
+        cellEditor.getTableCellEditorComponent(table, null, false, 0, OBJECT_COLUMN);
+    }
 
+//---------------- Sanity testing: StrictNumberFormatter - 
+    
     @Test (expected = ParseException.class)
     public void testNumberFormatter() throws ParseException {
         NumberFormat format = NumberFormat.getIntegerInstance();
@@ -155,6 +214,8 @@ public class NumberEditorExtTest extends InteractiveTestCase {
         String text = new Integer(Integer.MAX_VALUE).toString() + "1";
         formatter.stringToValue(text);
     }
+
+//---------- end sanity testing: StrictNumberFormatter
     
     /**
      * Issue #1183-swingx: NumberEditorExt throws in getCellEditorValue if
@@ -263,7 +324,60 @@ public class NumberEditorExtTest extends InteractiveTestCase {
     }
 
 //----------------- interactive
+
     
+    /**
+     * Issue #1293-swingx: NumberEditorExt removes Escape binding for all formatted text fields.
+     * 
+     * Here we assure that the fix for the issue doesn't effect the table edit - must
+     * pass-through to allow correct cancel processing (non-strict number editor).
+     */
+    public void interactiveEscapeNonStrict() {
+        JXTable table = new JXTable(this.table.getModel());
+        table.setDefaultEditor(Number.class, new NumberEditorExt(false));
+        JXFrame frame = wrapWithScrollingInFrame(table, "non-strict: escape must cancel edit");
+        addStatusMessage(frame, "edit number column, press escape, verify editingCanceled");
+        final CellEditorReport report = new CellEditorReport();
+        table.getDefaultEditor(Number.class).addCellEditorListener(report);
+        Action verify = new AbstractAction("verify cancelled") {
+            
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                int cancelledCount = report.getCanceledEventCount();
+                report.clear();
+                LOG.info("cancelled event count must be > 0, was: " + cancelledCount);
+            }
+        };
+        addAction(frame, verify);
+        show(frame);
+    }
+    
+    /**
+     * Issue #1293-swingx: NumberEditorExt removes Escape binding for all formatted text fields.
+     * 
+     * Here we assure that the fix for the issue doesn't effect the table edit - must
+     * pass-through to allow correct cancel processing (strict number editor).
+     */
+    public void interactiveEscapeStrict() {
+        JXTable table = new JXTable(this.table.getModel());
+        JXFrame frame = wrapWithScrollingInFrame(table, "strict: escape must cancel edit");
+        addStatusMessage(frame, "edit number column, press escape, verify editingCanceled");
+        final CellEditorReport report = new CellEditorReport();
+        table.getDefaultEditor(Number.class).addCellEditorListener(report);
+        Action verify = new AbstractAction("verify cancelled") {
+            
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                int cancelledCount = report.getCanceledEventCount();
+                report.clear();
+                LOG.info("cancelled event count must be > 0, was: " + cancelledCount);
+            }
+        };
+        addAction(frame, verify);
+        show(frame);
+    }
+    
+
     /**
      * Issue #1183-swingx: NumberEditorExt throws in getCellEditorValue if
      *   Integer (short, byte..) below/above min/max.
@@ -583,12 +697,14 @@ public class NumberEditorExtTest extends InteractiveTestCase {
     @Before
     @Override
     public void setUp() throws Exception {
-        DefaultTableModel model = new DefaultTableModel(5, 1) {
+        DefaultTableModel model = new DefaultTableModel(5, COLUMN_COUNT) {
 
             @Override
             public Class<?> getColumnClass(int columnIndex) {
                 if (columnIndex == INTEGER_COLUMN)
                     return Integer.class;
+                if (columnIndex == NUMBER_COLUMN) 
+                    return Number.class;
                 return super.getColumnClass(columnIndex);
             }
             
